#ifndef _SHEEP_COORDINATE_H_
#define _SHEEP_COORDINATE_H_

#include <stdint.h>
#include <stdbool.h>

#define SHEEP_COORDINATE_T_PLACE_LEN 20
#define SHEEP_COORDINATE_T_TIME_LEN 10

/*
 * Data structure to store basic information about a mower coordinate.
 */
#pragma pack(push)
#pragma pack(1)
typedef struct {
	uint32_t index;
	char place[SHEEP_COORDINATE_T_PLACE_LEN];
	char time[SHEEP_COORDINATE_T_TIME_LEN];
} sheep_coordinate_t;
#pragma pack(pop)
/**
 * Struct packing (see https://en.wikipedia.org/wiki/Data_structure_alignment#Typical_alignment_of_C_structs_on_x86)
 * is neccessary to make this implementation portable as some compilers
 * do align the data structures a little bit differently.
 * Our sheep database also uses this packing.
 */


/**
 * This function reads coordinates from the sheep's database.
 * @param filename The file name to read the coordinates from.
 * @param buf The buffer to read the coordinates into. This must be a valid (allocated) pointer.
 * @param bufsize The size of the buffer in bytes.
 * @param decrypt Determines whether the file has to be decrypted beforehand.
 * @return The number of read coordinates.
 */
int read_coordinates_from_file(const char *filename, const sheep_coordinate_t *buf, const size_t bufsize, bool decrypt);

/**
 * This function writes coordinates to a new sheep's database.
 * @param filename The file name to write the coordinates to.
 * @param buf The buffer of coordinates to write.
 * @param count The number of coordinates to write.
 * @param encrypt Determines whether the file has to be encrypted before writing.
 * @return The number of coordinates that actually have been written.
 */
int write_coordinates_to_file(const char *filename, const sheep_coordinate_t *buf, const size_t count, bool encrypt);

#endif
